@@ -151,9 +151,9 @@ static int *fds;	/* listening file descriptors (UNIX socket, inet domain sockets
 
 static int pcp_unix_fd; /* unix domain socket fd for PCP (not used) */
 static int pcp_inet_fd; /* inet domain socket fd for PCP */
-extern char pcp_conf_file[POOLMAXPATHLEN+1]; /* path for pcp.conf */
-extern char conf_file[POOLMAXPATHLEN+1];
-extern char hba_file[POOLMAXPATHLEN+1];
+extern char *pcp_conf_file; /* path for pcp.conf */
+extern char *conf_file;
+extern char *hba_file;
 
 static int exiting = 0;		/* non 0 if I'm exiting */
 static int switching = 0;		/* non 0 if I'm fail overing or degenerating */
@@ -193,6 +193,8 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	MemoryContext MainLoopMemoryContext;
 	sigjmp_buf	local_sigjmp_buf;
 
+	bool first = true;
+
 	/* For PostmasterRandom */
 	gettimeofday(&random_start_time, NULL);
 
@@ -310,6 +312,15 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 											  ALLOCSET_DEFAULT_INITSIZE,
 											  ALLOCSET_DEFAULT_MAXSIZE);
 
+	/*
+	 * if the primary node id is not loaded by watchdog, search for it
+	 */
+	if (Req_info->primary_node_id < 0)
+	{
+		/* Save primary node id */
+		Req_info->primary_node_id = find_primary_node_repeatedly();
+	}
+
 	/* fork a child for PCP handling */
 	pcp_unix_fd = create_unix_domain_socket(pcp_un_addr);
 	/* Add onproc exit to clean up the unix domain socket at exit */
@@ -327,17 +338,6 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 	retrycnt = 0;		/* reset health check retry counter */
 	sys_retrycnt = 0;	/* reset SystemDB health check retry counter */
 
-	/*
-	 * check for child signals to ensure child startup before reporting successfull start
-	 */
-	CHECK_REQUEST;
-
-	ereport(LOG,
-			(errmsg("%s successfully started. version %s (%s)", PACKAGE, VERSION, PGPOOLVERSION)));
-
-	/* Save primary node id */
-	Req_info->primary_node_id = find_primary_node();
-
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
 		/* Since not using PG_TRY, must reset error stack by hand */
@@ -417,6 +417,15 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 		bool all_nodes_healthy;
 		CHECK_REQUEST;
 
+		/*
+		 * check for child signals to ensure child startup before reporting
+		 * successfull start.
+		 */
+		if (first)
+			ereport(LOG,
+					(errmsg("%s successfully started. version %s (%s)", PACKAGE, VERSION, PGPOOLVERSION)));
+		first = false;
+
 		/* do we need health checking for PostgreSQL? */
 		if (pool_config->health_check_period > 0)
 		{
@@ -479,7 +488,7 @@ int PgpoolMain(bool discard_status, bool clear_memcache_oidmaps)
 			}
 			pool_sleep(pool_config->health_check_period);
 		}
-		else /* Health Check is not enable and we have not much to do */
+		else /* Health Check is not enabled and we have not much to do */
 		{
 			processState = SLEEPING;
 			for (;;)
