@@ -858,7 +858,11 @@ POOL_STATUS Parse(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend,
 
 		node = (Node *) lfirst(list_head(parse_tree_list));
 
-		insert_stmt_with_lock = need_insert_lock(backend, stmt, node);
+		/* If replication mode, check to see what kind of insert lock is
+		 * neccessary.
+		 */
+		if (REPLICATION)
+			insert_stmt_with_lock = need_insert_lock(backend, stmt, node);
 
 		/*
 		 * Start query context
@@ -1623,7 +1627,16 @@ POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 					 */
 					if (TSTATE(backend, MASTER_SLAVE ? PRIMARY_NODE_ID : REAL_MASTER_NODE_ID) == 'T')
 					{
-						pool_set_writing_transaction();
+						/* However, if the query is "SET TRANSACTION READ ONLY" or its variant,
+						 * don't set it.
+						 */
+						if (!pool_is_transaction_read_only(node))
+						{
+							ereport(DEBUG1,
+									(errmsg("not SET TRANSACTION READ ONLY")));
+
+							pool_set_writing_transaction();
+						}
 					}
 
 					/*
