@@ -5,7 +5,7 @@
 * pgpool: a language independent connection pool server for PostgreSQL
 * written by Tatsuo Ishii
 *
-* Copyright (c) 2003-2013	PgPool Global Development Group
+* Copyright (c) 2003-2015	PgPool Global Development Group
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby
@@ -91,9 +91,12 @@ POOL_CONNECTION *pool_open(int fd, bool backend_connection)
 	cp->sbufsz = 0;
 	cp->buf2 = NULL;
 	cp->bufsz2 = 0;
+	cp->buf3 = NULL;
+	cp->bufsz3 = 0;
 
 	cp->fd = fd;
-    
+	cp->socket_state = POOL_SOCKET_VALID;
+
     MemoryContextSwitchTo(oldContext);
 
 	return cp;
@@ -110,13 +113,15 @@ void pool_close(POOL_CONNECTION *cp)
 	if (!cp->isbackend)
 		shutdown(cp->fd, 1);
 	close(cp->fd);
-
+	cp->socket_state = POOL_SOCKET_CLOSED;
 	pfree(cp->wbuf);
 	pfree(cp->hp);
 	if (cp->sbuf)
 		pfree(cp->sbuf);
 	if (cp->buf2)
 		pfree(cp->buf2);
+	if (cp->buf3)
+		pfree(cp->buf3);
 	pool_discard_params(&cp->params);
 
 	pool_ssl_close(cp);
@@ -184,6 +189,7 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 				continue;
 			}
 
+			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
 				/* if fail_over_on_backend_error is true, then trigger failover */
@@ -205,18 +211,23 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			}
 			else
 			{
-                ereport(ERROR,
+                ereport(FRONTEND_ERROR,
 					(errmsg("unable to read data from frontend"),
                          errdetail("socket read failed with an error \"%s\"", strerror(errno))));
 			}
 		}
 		else if (readlen == 0)
 		{
-			cp->EOF_on_socket = true;
+			cp->socket_state = POOL_SOCKET_EOF;
 			if (cp->isbackend)
 			{
+				if(processType == PT_MAIN)
+					ereport(ERROR,
+						(errmsg("unable to read data from DB node %d",cp->db_node_id),
+							 errdetail("EOF encountered with backend")));
+
                 ereport(FATAL,
-                        (errmsg("unable to read data from DB node %d",cp->db_node_id),
+					(errmsg("unable to read data from DB node %d",cp->db_node_id),
                          errdetail("EOF encountered with backend")));
 
 #ifdef NOT_USED
@@ -315,6 +326,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 				continue;
 			}
 
+			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
 				/* if fail_over_on_backend_error is true, then trigger failover */
@@ -343,7 +355,7 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
 		}
 		else if (readlen == 0)
 		{
-			cp->EOF_on_socket = true;
+			cp->socket_state = POOL_SOCKET_EOF;
 			if (cp->isbackend)
 			{
                 ereport(ERROR,
@@ -593,7 +605,7 @@ int pool_flush(POOL_CONNECTION *cp)
                     (errmsg("unable to flush data to frontend"),
                          errdetail("pgpool is in replication mode, ignoring error to keep consistency among backends")));
 			else
-                ereport(ERROR,
+                ereport(FRONTEND_ERROR,
                     (errmsg("unable to flush data to frontend")));
 
 		}
@@ -767,6 +779,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 
 		if (readlen == -1)
 		{
+			cp->socket_state = POOL_SOCKET_ERROR;
 			if (cp->isbackend)
 			{
 				notice_backend_error(cp->db_node_id);
@@ -788,7 +801,7 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 			/*
 			 * just returns an error, not trigger failover or degeneration
 			 */
-			cp->EOF_on_socket = true;
+			cp->socket_state = POOL_SOCKET_EOF;
             ereport(ERROR,
                 (errmsg("unable to read data from %s",cp->isbackend?"backend":"frontend"),
                      errdetail("EOF read on socket")));
@@ -986,7 +999,7 @@ int pool_push(POOL_CONNECTION *cp, void *data, int len)
 	char *p;
 
 	ereport(DEBUG1,
-		(errmsg("flushing data of len: %d", len)));
+		(errmsg("pushing data of len: %d", len)));
 
 
     MemoryContext oldContext = SwitchToConnectionContext(cp->isbackend);
@@ -997,8 +1010,8 @@ int pool_push(POOL_CONNECTION *cp, void *data, int len)
 	}
 	else
 	{
-		p = cp->buf3 + cp->bufsz3;
 		cp->buf3 = repalloc(cp->buf3, cp->bufsz3 + len);
+		p = cp->buf3 + cp->bufsz3;
 	}
 
 	memcpy(p, data, len);
